function PadeShamPoly3(fx,gxQ,gxD,xRange,Lb,Ub,runNum,sFxName,diaryFilename)
%PADESHAMPOLY2 Summary of this function goes here
%   Detailed explanation goes here
clc
global bDeleteIfExists
global bUseDiary
global xdata
global ydata
global orderQ
global orderD 
global ggxQ
global ggxD

warning('off','all')
if isempty(sFxName)
  sFxName = getFuncName(fx);
end
xdata = xRange';
ydata = xdata;
for i=1:length(xdata)
  ydata(i)=fx(xdata(i));
end
ggxQ = gxQ;
ggxD = gxD;

xmin = min(xdata);
xmax = max(xdata);
ymin = min(ydata);
ymax = max(ydata);
xdata = (xdata - xmin)/(xmax - xmin) + 1;
ydata = (ydata - ymin)/(ymax - ymin) + 1;
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);


options = optimoptions('particleswarm', 'Display', 'iter');
[x,psAICc] = particleswarm(@optimFunc,8,Lb,Ub,options);
qA = x(1);
qB = x(2);
qC = x(3);
dA = x(4);
dB = x(5);
dC = x(6);
orderQ = round(x(7));
orderD = round(x(8));
if bUseDiary
  diaryFilename = strrep(diaryFilename, ".txt", strcat("_Q", num2str(orderQ),"_D", num2str(orderD),"_run", num2str(runNum),".txt"));
  if exist(diaryFilename, 'file')==2
    if bDeleteIfExists
      delete(diaryFilename);
    else
      return;
    end
  end
end
X = [];
for i=1:orderQ
  xs = xdata.^gxQ(i,qA,qB,qC);
  X = [X;xs']; 
end
for i=1:orderD
  xs = ydata.*(xdata.^gxD(i,dA,dB,dC));
  X = [X;xs']; 
end
X = X';
lm = fitlm(X,ydata);
if bUseDiary
  diary(diaryFilename)
end
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);
sp = getFuncName(gxQ);
fprintf('Shammas polynomial power for Q(x) is %s\n', sp);
sp = getFuncName(gxD);
fprintf('Shammas polynomial power for D(x) is %s\n', sp);
format long
disp(lm);
anva = anova(lm,'summary');
disp(anva);
format short
fprintf('for Q(x) A = %f, B = %f, C = %f\n', qA, qB, qC);
fprintf('for D(x) A = %f, B = %f, C = %f\n', dA, dB, dC);
fprintf('order Q(x) = %f, order D(x) = %f\n',orderQ, orderD);
fprintf("Xmin = %f and Xmax = %f\n", xmin, xmax);
fprintf("Ymin = %f and Ymax = %f\n", ymin, ymax);
lstPwrQ = zeros(orderQ,1);
for i=1:orderQ
  lstPwrQ(i) = gxQ(i,qA,qB,qC);
end
lstPwrD= zeros(orderD,1);
for i=1:orderD
  lstPwrD(i) = gxD(i,dA,dB,dC);
end
fprintf('List of powers for Q(x): [');
for i=1:orderQ-1
 fprintf('%f, ', lstPwrQ(i));
end
fprintf('%f]\n', lstPwrQ(orderQ));
fprintf('List of powers for D(x): [');
for i=1:orderD-1
 fprintf('%f, ', lstPwrD(i));
end
fprintf('%f]\n', lstPwrD(orderD));
fprintf('Fitting %s in range (%f, %f)\n', sFxName, min(xdata),max(xdata));
n = length(xdata);
sumsqr = 0;
for i=1:n
  qx = lm.Coefficients{1,1};
  for j=2:+orderQ+1
    qx = qx + lm.Coefficients{j,1} * xdata(i)^lstPwrQ(j-1);
  end
  dx = 1;
  k = 1;
  for j=orderQ+2:orderQ+orderD+1
    dx = dx - lm.Coefficients{j,1} * xdata(i)^lstPwrD(k);
    k = k + 1;
  end
  yc = qx/dx;
  sumsqr = sumsqr + (ydata(i) - yc)^2;
end  
k = orderQ + orderD + 1;
fprintf('MSS of errors squared = %e\n', sqrt(sumsqr)/n);
fprintf('Corrected MSS of errors squared = %e\n', sqrt(sumsqr)/n*sqrt(6));
fprintf('R-squared = %12.8f\n', lm.Rsquared.Ordinary);
fprintf('R-squared Adjusted = %12.8f\n', lm.Rsquared.Adjusted);
AIC = lm.ModelCriterion.AIC;
AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
fprintf('Particle swarm AICc =  %e\n', psAICc);
fprintf('AIC =  %e\n', AIC);
fprintf('AICc =  %e\n', AICc);

if bUseDiary
  diary off
end
end

function AICc = optimFunc(x)
  global xdata
  global ydata
  global orderQ
  global orderD
  global ggxQ
  global ggxD
  
  qA = x(1);
  qB = x(2);
  qC = x(3);
  dA = x(4);
  dB = x(5);
  dC = x(6);
  orderQ = round(x(7));
  orderD = round(x(8));
  X = [];
  for i=1:orderQ
    xs = xdata.^ggxQ(i,qA,qB,qC);
    X = [X;xs'];
  end
  for i=1:orderD
    xs = ydata.*(xdata.^ggxD(i,dA,dB,dC));
    X = [X;xs'];
  end  
  X = X';
  lm = fitlm(X,ydata);

  n = length(xdata);
  k = orderQ + orderD + 1;
  AIC = lm.ModelCriterion.AIC;
  AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
  if isinf(AICc), AICc = -1e+99; end
end

function sFx = getFuncName(fx)
  sFx = func2str(fx);
  if sFx(1:2)=="@("
    i = strfind(sFx,")");
    sFx = sFx(i(1)+1:end);
  elseif sFx(1)=="@"
    sFx = strcat(sFx(2:end),".m");
  else
    % return sFx as is
  end
end